using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using System.IO;

/*
 * PRIMEVAL LABORATORIES
 * 
 * Created by Brian MacIntosh in 48 hours for the Ludum Dare Competition
 * 
 * Compo #24 : Evolution
 * 
 * Started: 8/24/12
 * 
 */

namespace Primeval
{
    public enum MenuState
    {
        MAINMENU,
        GAME,
        PAUSE
    }

    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        private static GraphicsDeviceManager graphics;
        private SpriteBatch spriteBatch;

        public static MenuState currentState;

        public static Random random;
        public static ContentManager content;

        public static Player player;

        public static SpriteFont readoutFont;
        public static SpriteFont exportsFont;
        public static Texture2D pixel;
        private static Texture2D field;
        private static Texture2D fieldbase;
        private static Texture2D fieldlid_on;
        private static Texture2D fieldlid_off;
        private static Texture2D fieldlid
        {
            get
            {
                return (GameState.CurrentState == State.BATTLE ? fieldlid_on : fieldlid_off);
            }
        }
        private static Texture2D cursor;
        private static Texture2D crosshair;

        public static Vector2 botSpawnPoint;
        public static Vector2 fieldCenter;
        public static Vector2 playerSpawnPoint;

        public static Rectangle map;

        public static float transitionPosition;

        private static AudioEngine audioengine;
        private static WaveBank wavebank;
        public static SoundBank soundbank;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            if (!Directory.Exists("Exports")) Directory.CreateDirectory("Exports");

            graphics.PreferredBackBufferWidth = 1024;
            graphics.PreferredBackBufferHeight = 768;
#if !DEBUG
            graphics.IsFullScreen = true;
#endif

            botSpawnPoint = TransformToData(new Vector2(512, 235));
            fieldCenter = TransformToData(new Vector2(512, 460));
            playerSpawnPoint = TransformToData(new Vector2(512, 500));

            //Calculate borders of map
            Vector2 mapZero = TransformToData(Vector2.Zero);
            Vector2 mapFar = TransformToData(new Vector2(1024, 768));
            map = new Rectangle(
                (int)map.X,
                (int)map.Y,
                (int)mapFar.X + (int)map.X,
                (int)mapFar.Y + (int)map.Y);

            random = new Random();
            content = Content;
            player = new Player();
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            currentState = MenuState.MAINMENU;

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            audioengine = new AudioEngine(AppDomain.CurrentDomain.BaseDirectory + content.RootDirectory + "/Win/xact.xgs");
            wavebank = new WaveBank(audioengine, AppDomain.CurrentDomain.BaseDirectory + content.RootDirectory + "/Win/Wave Bank.xwb");
            soundbank = new SoundBank(audioengine, AppDomain.CurrentDomain.BaseDirectory + content.RootDirectory + "/Win/Sound Bank.xsb");

            readoutFont = content.Load<SpriteFont>("readout");
            exportsFont = content.Load<SpriteFont>("exportsfont");

            pixel = new Texture2D(graphics.GraphicsDevice, 1, 1);
            pixel.SetData<Color>( new Color[] { Color.White } );

            field = content.Load<Texture2D>("field");
            fieldbase = content.Load<Texture2D>("fieldbase");
            fieldlid_on = content.Load<Texture2D>("fieldlid_go");
            fieldlid_off = content.Load<Texture2D>("fieldlid_stop");
            crosshair = content.Load<Texture2D>("crosshair");
            cursor = content.Load<Texture2D>("cursor");

            Player.LoadContent();
            Projectile.LoadContent();
            Bot.LoadContent();
            Particle.LoadContent();

            Laser.LoadContent();
            Rocket.LoadContent();

            MainMenu.LoadContent();
            PauseMenu.LoadContent();
            GenePool.LoadContent();
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            audioengine.Update();

            //Pause
            if ((player.Die && MessageDisplay.Waiting && currentState == MenuState.GAME)
                || Keyboard.GetState().IsKeyDown(Keys.Escape)
                || GamePad.GetState(PlayerIndex.One).IsButtonDown(Buttons.Start))
                currentState = MenuState.PAUSE;

            switch (currentState)
            {
                case MenuState.MAINMENU:
                    MainMenu.Update(gameTime);
                    break;
                case MenuState.PAUSE:
                    if (transitionPosition > 0)
                    {
                        transitionPosition -= (float)gameTime.ElapsedGameTime.TotalSeconds / 0.1f;
                        if (transitionPosition < 0) transitionPosition = 0;
                    }
                    PauseMenu.Update(gameTime);
                    break;
                case MenuState.GAME:
                    if (transitionPosition > 0)
                    {
                        transitionPosition -= (float)gameTime.ElapsedGameTime.TotalSeconds / 0.1f;
                        if (transitionPosition < 0) transitionPosition = 0;
                    }
                    GameState.Update(gameTime);
                    Bot.UpdateAll(gameTime);
                    player.Update(gameTime);
                    Projectile.UpdateAll(gameTime);
                    Particle.UpdateAll(gameTime);
                    break;
            }

            MessageDisplay.Update(gameTime);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            spriteBatch.Begin(
                SpriteSortMode.Deferred,
                BlendState.NonPremultiplied);

            switch (currentState)
            {
                case MenuState.MAINMENU:
                    MainMenu.Draw(spriteBatch);
                    break;
                case MenuState.PAUSE:
                case MenuState.GAME:
                    spriteBatch.Draw(field, Vector2.Zero, Color.White);
                    spriteBatch.Draw(
                        fieldbase,
                        TransformToDraw(botSpawnPoint) - new Vector2(fieldbase.Width, fieldbase.Height) / 2,
                        Color.White);

                    Bot.DrawAll(spriteBatch);
                    Particle.DrawAll(spriteBatch);
                    player.Draw(spriteBatch);
                    Projectile.DrawAll(spriteBatch);
                    MessageDisplay.Draw(spriteBatch);
                    GenePool.Draw(spriteBatch);

                    spriteBatch.Draw(fieldlid, new Vector2(512, 100) - new Vector2(fieldlid.Width / 2, 0), Color.White);

                    Bot.DrawAllUI(spriteBatch);
                    GameState.Draw(spriteBatch);

                    if (currentState == MenuState.PAUSE)
                        PauseMenu.Draw(spriteBatch);
                    break;
            }

            spriteBatch.Draw(
                (currentState == MenuState.GAME ? crosshair : cursor),
                new Vector2(Mouse.GetState().X, Mouse.GetState().Y)
                - new Vector2(crosshair.Width, crosshair.Height) / 2,
                Color.White);

            if (transitionPosition > 0)
            {
                spriteBatch.Draw(pixel,
                    new Rectangle(0, 0, 1024, 768),
                    new Color(0, 0, 0, transitionPosition));
            }

            spriteBatch.End();

            base.Draw(gameTime);
        }

        public static void Reset()
        {
            Bot.Reset();
            Projectile.Reset();
            player.Reset();
            Particle.Reset();
            MessageDisplay.Clear();
            GenePool.Reset();
            GameState.Reset();
        }

        /// <summary>
        /// Used to simulate a slightly frontward (not top-down) view
        /// </summary>
        #region Transform
        private const float isoFactor = 0.6f;

        public static Vector2 TransformToDraw(Vector2 pos)
        {
            pos.Y = (pos.Y - graphics.PreferredBackBufferHeight / 2) * isoFactor
                + graphics.PreferredBackBufferHeight / 2;
            return pos;
        }

        public static Vector2 TransformToData(Vector2 pos)
        {
            pos.Y = (pos.Y - graphics.PreferredBackBufferHeight / 2) / isoFactor
                + graphics.PreferredBackBufferHeight / 2;
            return pos;
        }
        #endregion
    }
}
