﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Primeval
{
    static class MessageDisplay
    {
        private static List<string> messages;

        private static Rectangle frame;
        private static Rectangle textframe;

        private static int currentLine;
        private static int currentChar;

        private static TimeSpan charTime = TimeSpan.FromMilliseconds(70);
        private static TimeSpan currentCharTime;

        private static TimeSpan lineWaitTime = TimeSpan.FromMilliseconds(600);
        private static TimeSpan currentLineWaitTime;

        private static bool cursorState;
        private static TimeSpan cursorFlashTime = TimeSpan.FromMilliseconds(250);
        private static TimeSpan currentCursorFlashTime;

        private static float LineHeight { get { return Game1.readoutFont.MeasureString("Hj").Y; } }

        private static int LineCapacity
        {
            get
            {
                return (int)(textframe.Height / LineHeight);
            }
        }

        static MessageDisplay()
        {
            messages = new List<string>();

            frame = new Rectangle(
                30, 50, 300, 170);
            textframe = new Rectangle(
                frame.X + 10, frame.Y + 10, frame.Width - 20, frame.Height - 20);
        }

        public static void PostMessage(string message)
        {
            messages.Add(message);
        }

        public static void Clear()
        {
            PostMessage("\t");
        }

        private static bool advanceLine;

        public static void Update(GameTime gameTime)
        {
            currentCharTime -= gameTime.ElapsedGameTime;
            currentLineWaitTime -= gameTime.ElapsedGameTime;
            if (currentCharTime <= TimeSpan.Zero
                && currentLineWaitTime <= TimeSpan.Zero
                && messages.Count > 0)
            {
                if (advanceLine)
                {
                    currentChar = 0;
                    currentLine++;
                    if (currentLine >= LineCapacity)
                    {
                        messages.RemoveAt(0);
                        currentLine--;
                    }
                    advanceLine = false;
                }

                //Read "clear" signal
                if (messages[currentLine] == "\t")
                {
                    messages.RemoveRange(0, currentLine + 1);
                    currentChar = 0;
                    currentLine = 0;
                }
                else
                {
                    //Advance cursor
                    currentChar++;
                    if (currentChar >= messages[currentLine].Length)
                    {
                        if (messages.Count - 1 > currentLine)
                        {
                            advanceLine = true;
                            currentLineWaitTime = lineWaitTime;
                        }
                        else
                            currentChar--;
                    }
                }
            }

            //Blink cursor
            currentCursorFlashTime -= gameTime.ElapsedGameTime;
            if (currentCursorFlashTime < TimeSpan.Zero)
            {
                cursorState = !cursorState;
                currentCursorFlashTime = cursorFlashTime;
            }
        }

        public static bool Waiting
        {
            get
            {
                return messages.Count == 0 ||
                    (currentLine == messages.Count - 1
                    && currentChar == messages[currentLine].Length - 1);
            }
        }

        public static void Draw(SpriteBatch spriteBatch)
        {
            //Draw caption
            spriteBatch.DrawString(
                Game1.readoutFont,
                "Console",
                new Vector2(
                    frame.Center.X - Game1.readoutFont.MeasureString("Console").X / 2, 20),
                Color.White);

            //Draw background box
            spriteBatch.Draw(
                Game1.pixel,
                frame,
                new Color(0.8f, 0.8f, 0.8f, 0.3f));

            //Draw whole lines
            for (int c = 0; c < currentLine; c++)
            {
                spriteBatch.DrawString(
                    Game1.readoutFont,
                    messages[c],
                    new Vector2(textframe.Left, textframe.Top + c * LineHeight),
                    Color.White);
            }

            //Draw partial line
            if (messages.Count > 0)
            {
                string drawstring = messages[currentLine].Substring(0,
                    (int)Math.Min(messages[currentLine].Length, currentChar));
                if (cursorState) drawstring += "_";
                spriteBatch.DrawString(
                    Game1.readoutFont,
                    drawstring,
                    new Vector2(textframe.Left, textframe.Top + currentLine * LineHeight),
                    Color.White);
            }
        }

        public static string PadNum(int num, int length)
        {
            string str = num.ToString();
            for (int c = 0; c < length - str.Length; c++) str = "0" + str;
            return str;
        }
    }
}
